#!/usr/bin/perl
#
# Description:
#   The script reads a gpx waypoint file and parses for
#   usefull information to be stored into an WPT file (waypoint, POI)
#
# History:
#   0.2 - first version
#       released: 04.08.2010
#   0.3 - fix several utf8 issues
#       - change from nw+se area to gpx track specified area (--area)
#       - allow GPS device storage path specification (--wpt_path|wp)
#       - use file name as local destination directory
#       - add picture if available
#       released: 05.08.2010
#
#
# $Revision:  $       (c) E. Leschke                       $Date: 05.08.2010$
#-----------------------------------------------------------------------------
my $version = "0.3";
my $DEBUG = 0;
 
use utf8;
use strict;
use warnings;
use English;
use Getopt::Long;
use Data::Dumper;
use File::Basename;
use Pod::Usage;
use LWP::Simple;
use Geo::Gpx;
use Geo::Coordinates::UTM;
use File::Slurp;
use Math::Polygon;
use LWP::Simple;


#=====================
# Start configuration
#=====================
our $WptPath = "\\Storage Card\\TwoNavData\\Data\\WPT";  # default path on GPS device for WPT, txt, jpg files

#OS Specific Variables
my $linuxDirDivider = "\/"; my $linuxMoveCommand = "mv"; my $linuxDeleteCommand = "rm"; my $linuxCopyCommand = "cp";
my $windowsDirDivider = "\\"; my $windowsMoveCommand = "move"; my $windowsDeleteCommand = "del"; my $windowsCopyCommand = "copy";
#=====================
# End configuration
#=====================

my ($systemDirDivider, $systemMoveCommand, $systemCopyCommand, $systemDeleteCommand);

if($^O ge "linux" || $^O ge "cygwin" )
{
  $systemDirDivider = $linuxDirDivider;
  $systemMoveCommand = $linuxMoveCommand;
  $systemCopyCommand = $linuxCopyCommand;
  $systemDeleteCommand = $linuxDeleteCommand;
}
elsif($^O ge "MSWin")
{
  $systemDirDivider = $windowsDirDivider;
  $systemMoveCommand = $windowsMoveCommand;
  $systemCopyCommand = $windowsCopyCommand;
  $systemDeleteCommand = $windowsDeleteCommand;
 }
if($^O eq "darwin")
{
  $systemDirDivider = $linuxDirDivider;
  $systemMoveCommand = $linuxMoveCommand;
  $systemDeleteCommand = $linuxDeleteCommand;
}



# forward declaration
sub WriteWpt($); 
 
 
print( "\n" );
print( "----------------------------------------------------------\n" );
print( "|  $0  v$version \n" );
print( "----------------------------------------------------------\n" );

# Check for valid number of arguments
if (($#ARGV < 0))  { pod2usage(2); }

# option handling
our ($help, $man, $debug) = (0,0,0);
our ($GpxFileName, $NorthWest, $SouthEast, $AreaGpxFileName);
our $ForceCmt = 0;  # no comment in wpt, all in txt file

GetOptions(
  'help|h|?'          => \$help,
  'man'               => \$man,
  'debug=i'           => \$debug,
  'gpx=s'             => \$GpxFileName,
  'nw=s'              => \$NorthWest,
  'se=s'              => \$SouthEast,
  'area=s'            => \$AreaGpxFileName,
  'wpt_path|wp=s'     => \$WptPath,
  'cmt'               => \$ForceCmt,
          );

pod2usage(1) if $help;
pod2usage(-verbose => 2) if $man;
$DEBUG=$debug;

#-------------------------------------------------------------------

          
my ($NorthWest_lat, $NorthWest_lon, $SouthEast_lat, $SouthEast_lon);
my $poly;

if ((defined $NorthWest) && (defined $SouthEast))
{
  $NorthWest =~ m/(.+),(.+)/;
  $NorthWest_lat = $1;
  $NorthWest_lon = $2;

  $SouthEast =~ m/(.+),(.+)/;
  $SouthEast_lat = $1;
  $SouthEast_lon = $2;

  $poly = Math::Polygon->new( [$NorthWest_lat,$NorthWest_lon],
                              [$SouthEast_lat,$NorthWest_lon],
                              [$SouthEast_lat,$SouthEast_lon],
                              [$NorthWest_lat,$SouthEast_lon],
                              [$NorthWest_lat,$NorthWest_lon]
                            );
}

#-------------------------------------------------------------------
# handling of track defined area
#-------------------------------------------------------------------
my ($AreaGpxContent, @AreaPoints, @PointArr, @StartPoint);

if (defined $AreaGpxFileName)
{
  $AreaGpxContent = read_file($AreaGpxFileName); # entire file in scalar
  my $AreaGpx  = Geo::Gpx->new( xml => $AreaGpxContent );

  my $iter = $AreaGpx->iterate_trackpoints();
  my $i = 1;
  # for each of the track points
  while ( my $TrkPt = $iter->() ) 
  {
    @PointArr = [$TrkPt->{lat}, $TrkPt->{lon}];
    if ($i == 1)  { @StartPoint = @PointArr; }
    $i++;

    push @AreaPoints, @PointArr;
  }
  push @AreaPoints, @StartPoint;  # close the polygon
  if ($DEBUG>0) { print "AreaPoints:\n", Dumper(@AreaPoints); }
  $poly = Math::Polygon->new(points => \@AreaPoints);
}

#-------------------------------------------------------------------
#  open and parse the gpx file, open wpt file
#-------------------------------------------------------------------
print( "Start parsing gpx file $GpxFileName\n" );

unless (-e $GpxFileName) { die "Can't find GPX file: $GpxFileName"; }

my $GpxContent = read_file($GpxFileName); # entire file in scalar

my $Basename = basename($GpxFileName, '.gpx');
my $DEST_DIR = $Basename;
# create destination dir if not exist
unless (-d $DEST_DIR)
{
  system("mkdir $DEST_DIR");
}
my $WptFile = $DEST_DIR . $systemDirDivider . $Basename . ".wpt";
open( WPT, ">:utf8", $WptFile ) or die "Can't open WPT file: $!";

print WPT << "EOH";
B  UTF-8
G  WGS 84
U  1
EOH

# Parse GPX
my $gpx  = Geo::Gpx->new( xml => $GpxContent );


#-------------------------------------------------------------------
#  write waypoints into wpt file
#-------------------------------------------------------------------
print( "writing wpt file $WptFile\n" );
my $count = 0;
my $inside = 0;
my $bounds = $gpx->bounds();
#  $bounds = {
#    minlat => 57.120939,
#    minlon => -2.9839832,
#    maxlat => 57.781729,
#    maxlon => -1.230902
#  };

# for each of the waypoints
my $iter = $gpx->iterate_waypoints();
my (@point);

while ( my $wpt = $iter->() ) 
{
  @point = [$wpt->{lat}, $wpt->{lon}];
    if ($DEBUG>0) { print "WptPoint:\n", Dumper(@point); }
  
  if (defined $poly)  # an area was specified
  {
    if ($poly->contains( @point ))
    {
      $inside++;
      WriteWpt( $wpt );
    }
  }
  else  # all to be processed
  {
    $inside++;
    WriteWpt( $wpt );
  }
  
  $count++;
}

close WPT;

print "\ndone ($count entries processed, $inside entries inside and written).\n";




####################################################################
#  s u b r o u t i n e s
####################################################################

# ---------------------------------------
# WriteWpt 
# ---------------------------------------
sub WriteWpt($)
{
  my ($wpt) = @_;
  my ($name, $latitude, $longitude, $elevation, $comment, $link, $link_txt, $link_href, $sym);
  my ($id, $InfoFile);
  my ($page, $PicUrl, $Pic);

  if ($DEBUG>0) { print "Waypoint:\n", Dumper($wpt); }
  
  $name = $$wpt{name};
  $latitude = $$wpt{lat};
  $longitude = $$wpt{lon};
  $elevation = $$wpt{ele};
  if (!defined $elevation) { $elevation = 0; }
  $comment = $$wpt{cmt};
  $sym = $$wpt{sym};
  $link = $$wpt{link};
  $link_txt = $$link{text};
  $link_href = $$link{href};
  
  $link_href =~ m/http(.*)huette=(\d+)/;
  $id = $2;
  
  if ($DEBUG>0) { print "id=$id: lat=${latitude}, lon=${longitude}, ele=${elevation}\n"; }
  
  # write the additional info text file
  $InfoFile = $DEST_DIR . $systemDirDivider . "${id}.txt";
  open( INFO, ">:utf8", $InfoFile ) or die "Can't open WPT INFO file: $!";
  print INFO "Hütte: $name\015\012";
  print INFO "lat=${latitude}, lon=${longitude}, ele=${elevation}\015\012";
  print INFO "$comment\015\012";
  print INFO "$link_href\015\012";
  close INFO;

  # html format in case a picture is present 
  # found one here: http://www.tourenwelt.info/huettenliste/huette.php?huette=10524
  # big picture at: http://www.tourenwelt.info/huettenliste/img/huette/g/2391.jpg
  # use this for testing:  $page = get("http://www.tourenwelt.info/huettenliste/huette.php?huette=10524");

  # get and store a picture if available
  $page = get($link_href);
  if (defined $page)
  {
    if ($page =~ /Galerie:(.*?)<img src\=\"(http:\/\/www\.tourenwelt\.info\/huettenliste\/img\/huette.*?)\" /sg)
    { 
      $PicUrl = $2; 
      $PicUrl =~ s/\/k\//\/g\//;
      if ($DEBUG>0) { print "PicUrl=$PicUrl\n"; }
      $Pic = get($PicUrl);
      open( PIC, ">${DEST_DIR}${systemDirDivider}${id}.jpg" ) or die "Can't open WPT PIC file: $!";
      binmode( PIC );
      print PIC $Pic;
      close PIC;
    }
  }
   
  # convert into right format of wpt
  if ($latitude >= 0)
  { $latitude .= 'ºN'; }
  else
  { $latitude = abs($latitude); $latitude .= 'ºS'; }
  if ($longitude >= 0)
  { $longitude .= 'ºE'; }
  else
  { $longitude = abs($longitude); $longitude .= 'ºW'; }

  # write the waypoint
  print WPT "W  $name A ${latitude} ${longitude} 27-MAR-62 00:00:00 ${elevation}";
  if ($ForceCmt)  { print WPT " ${comment}\n"; } else { print WPT "\n"; }
  print WPT "w ${sym},0,0.0,16316664,248,1,39,${link_href},100.0,,-1,0\n";
  print WPT "a ${WptPath}\\${id}.txt,0\n";

  if (defined $Pic)  # there is a picture
  {
    print WPT "a ${WptPath}\\${id}.jpg,0\n";
  }
  
}



#==============================
# End of script
#==============================

__END__



#^L
#==============================
# POD follows
#==============================

=head1 NAME

B<bb2gpx>

The script converts GPX file into WPT file including linked info in text file.

=head1 SYNOPSIS

B<bb2gpx [options]>

  Options:
    --help | -h | -?            brief help message
    --man                       full documentation
    --gpx=<gpx_file>            file name of the input gpx file to be converted (mandatory)
    --nw=s                      north west point of the relevant area (--nw=lat,lon)  ***DEPRECATED***
    --se=s                      south east point of the relevant area (--se=lat,lon)  ***DEPRECATED***
    --area=<track.gpx>          file name of gpx track for the area of interrest
    --wpt_path|wp=<path>        path on the GPS device where the waypoint file will be stored
    --cmt                       force to write the gpx comment as wpt comment

=head1 OPTIONS

=over 8

=item --help | -h | -?

Print a brief help message and exits.

=item --man

Prints the manual page and exits.

=item --gpx

Specifies the GPX file as input. The resulting WPT file will have the same basename.


=item --nw=s   --se=s  ***DEPRECATED***

Specifies the area of interest as follow:

--nw	north west point of the relevant area (--nw=lat,lon)

--se	south east point of the relevant area (--se=lat,lon)


=item --area=<track.gpx>

Specifies the area of interest with the help of a track around 
The format of the track must be gpx, the polygon will be closed automatically from the last point to the first point


=item --wpt_path|wp

Specifies the path on the GPS device where the waypoint file together with the associated
txt and jpg files are to be stored.

default: "\Storage Card\TwoNavData\Data\WPT"

=item --cmt

Force to write the gpx comment as wpt comment (default: comment is only written into associated txt file)

   
=back

=head1 DESCRIPTION

   The script reads the given GPX file with waypoint inside and parses for
   usefull information to be stored into an WPT file (waypoint, POI).

=head1 PREREQUISITES

This tool is entirly written in perl and requires the following Perl modules installed:

  - English
  - Data::Dumper
  - Getopt::Long
  - Pod::Usage
  - File::Basename
  - LWP::Simple
  - HTML::FormatText
  - Geo::Gpx
  - Geo::Coordinates::UTM
  - File::Slurp
  - Math::Polygon
  - LWP::Simple


=head1 AUTHORS

This version of the tool is completely written by Ekkart Leschke.

=head1 COPYRIGHT

Copyright (c) 2010 Ekkart Leschke. All rights reserved.
This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.
See the Perl Artistic.txt for details.

=cut


#--end--
